from __future__ import print_function

import json
import os
import sys


class Container(dict):
    __slots__ = "__protected__",

    def __init__(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)
        self.__protected__ = False

    def __getattr__(self, item):
        if item in self:
            return self[item]
        raise AttributeError("(getattr) Attribute or key '{}' not in {}"
                             .format(item, self.__class__))

    def __setattr__(self, key, value):
        if key in self.__slots__:
            return super(self.__class__, self).__setattr__(key, value)

        if self.__protected__:
            raise AttributeError("(setattr) Attribute or key '{}' not in protected {}"
                                 .format(key, self.__class__))
        self[key] = value

    def as_json(self):
        """
        Return Containers content in json-format.
        """
        return json.dumps(self)

    def write(self, out=sys.stdout, overwrite=False):
        """
        Write Container content to file (or stdout by default)
        in json-format.
        """
        closeflag = False
        if isinstance(out, basestring):
            if not overwrite and os.path.exists(out):
                raise IOError("Can't overwrite: '{}'"
                              .format(out))
            out = open(out, "w")
            closeflag = True

        print(self.as_json(), file=out)
        if closeflag:
            out.close()

    def protect(self):
        """
        Don't allow new keys.
        """
        self.__protected__ = True

    def unprotect(self):
        """
        Protection dismissed.
        """
        self.__protected__ = False


if __name__ == "__main__":
    myContainer = Container()
    myContainer = Container(abc=123, efg=432)
    print(myContainer)
    # prints: {'abc': 123, 'efg': 432}

    myContainer.value1 = "New Data"
    myContainer.value2 = "More Data"
    print(myContainer)
    # prints: {'abc': 123, 'value2': 'More Data', 'value1': 'New Data', 'efg': 432}

    myContainer.protect()
    # no more new keys allowed
    myContainer.xyz = "what now?"
    # results in AttributeError
